import pandas as pd

from code_genie import Genie


def test_math(client):
    # use genie to get a method to add 2 numbers
    genie = Genie(data=[10, 20, 30], client=client)

    # call the method
    assert genie.plz(instructions="add all inputs").result == 60
    assert genie.plz(instructions="multiply all inputs").result == 6000


def test_math_additional_inputs(client):
    # use genie to get a method to add 2 numbers
    genie = Genie(data=4, client=client, copy_data_before_use=False)

    # call the method
    additional_input = {"y": 2}
    assert genie.plz(instructions="add data and y", additional_inputs=additional_input).result == 6
    assert genie.plz(instructions="multiply data and y", additional_inputs=additional_input).result == 8


def test_pd_mean(client):
    df = pd.DataFrame({"x": [1, 2, 3, 1, 2, 3], "y": [4, 5, 6, 4, 5, 6]})
    genie = Genie(data=df, client=client)

    # call the method
    assert genie.plz(instructions="sum of mean of x and y").result == 7
    assert set(genie.plz(instructions="distinct values of x").result) == {1, 2, 3}


def test_cache(client):
    # use genie to get a method to add 2 numbers
    genie = Genie(data=[1, 2, 3, 4], client=client)

    # call the method
    gr_1 = genie.plz(instructions="add all elements of input")
    assert gr_1.result == 10

    gr_2 = genie.plz(instructions="add all elements of input")
    assert gr_2.result == 10
    assert gr_1.id == gr_2.id

    gr_3 = genie.plz(instructions="add all elements of input", override=True)
    assert gr_3.result == 10
    assert gr_3.id != gr_1.id


def test_pd_copy_before_use(client):
    df = pd.DataFrame({"x": [1, 2, 3, 1, 2, 3], "y": [4, 5, 6, 4, 5, 6]})
    genie = Genie(data=df, client=client)

    # call the method
    gr = genie.plz(instructions="drop column x")
    assert set(gr.result.columns) == {"y"}
    assert set(df.columns) == {"x", "y"}
